@@ -561,6 +561,7 @@ class Field(mongoengine.fields.BaseField):
         info=None,
         read_only=False,
         created_at=None,
+        schema=None,
         **kwargs,
     ):
         super().__init__(**kwargs)
@@ -568,6 +569,7 @@ class Field(mongoengine.fields.BaseField):
         self._info = info
         self._read_only = read_only
         self._created_at = created_at
+        self._schema = schema
 
         self.__dataset = None
         self.__path = None
@@ -685,6 +687,15 @@ class Field(mongoengine.fields.BaseField):
     def read_only(self, read_only):
         self._read_only = read_only
 
+    @property
+    def schema(self):
+        """ """
+        return self._schema
+
+    @schema.setter
+    def schema(self, schema):
+        self._schema = schema
+
     @property
     def created_at(self):
         """The datetime the field was created."""
