@@ -70,6 +70,7 @@ class SampleFieldDocument(EmbeddedDocument):
     description = StringField(null=True)
     info = DictField(null=True)
     read_only = BooleanField(default=False)
+    schema = DictField(null=True)
     created_at = DateTimeField(null=True, default=None)
 
     def _set_created_at(self, created_at):
@@ -108,6 +109,7 @@ class SampleFieldDocument(EmbeddedDocument):
             description=self.description,
             info=self.info,
             read_only=self.read_only,
+            schema=self.schema,
             created_at=self.created_at,
         )
 
@@ -137,6 +139,7 @@ class SampleFieldDocument(EmbeddedDocument):
             description=field.description,
             info=field.info,
             read_only=field.read_only,
+            schema=field.schema,
             created_at=field.created_at,
         )
 
