@@ -1,7 +1,7 @@
 """
 Dataset run documents.
 
-| Copyright 2017-2023, Voxel51, Inc.
+| Copyright 2017-2025, Voxel51, Inc.
 | `voxel51.com <https://voxel51.com/>`_
 |
 """
@@ -29,5 +29,5 @@ class RunDocument(Document):
     version = StringField()
     timestamp = DateTimeField()
     config = DictField()
-    view_stages = ListField(StringField())
+    view_stages = ListField(StringField(), default=None)
     results = FileField()
