@@ -186,13 +186,6 @@ class MongoDelegatedOperationRepo(DelegatedOperationRepo):
                 )
             )
 
-        if "dataset_id_1" not in index_names:
-            indices_to_create.append(
-                IndexModel(
-                    [("dataset_id", pymongo.ASCENDING)], name="dataset_id_1"
-                )
-            )
-
         if "parent_id_1" not in index_names:
             indices_to_create.append(
                 IndexModel(
@@ -201,6 +194,18 @@ class MongoDelegatedOperationRepo(DelegatedOperationRepo):
                 )
             )
 
+        if "dataset_id_1_parent_id_1_scheduled_at_1" not in index_names:
+            indices_to_create.append(
+                IndexModel(
+                    [
+                        ("dataset_id", pymongo.ASCENDING),
+                        ("parent_id", pymongo.ASCENDING),
+                        ("scheduled_at", pymongo.DESCENDING),
+                    ],
+                    name="dataset_id_1_parent_id_1_scheduled_at_1",
+                )
+            )
+
         if indices_to_create:
             self._collection.create_indexes(indices_to_create)
 
