@@ -5,13 +5,14 @@ FiftyOne operators.
 | `voxel51.com <https://voxel51.com/>`_
 |
 """
-from .operator import Operator, OperatorConfig
+from .operator import Operator, OperatorConfig, PipelineOperator
 from .registry import (
     OperatorRegistry,
     get_operator,
     list_operators,
     operator_exists,
 )
+from .constants import ViewTarget
 from .evaluation_metric import EvaluationMetricConfig, EvaluationMetric
 from .executor import (
     execute_operator,
