@@ -8,9 +8,11 @@ FiftyOne operator execution.
 
 import asyncio
 import collections
+import dataclasses
 import inspect
 import logging
 import traceback
+from typing import Optional
 
 import fiftyone as fo
 import fiftyone.core.dataset as fod
@@ -298,9 +300,9 @@ async def execute_or_delegate_operator(
                 ctx.num_distributed_tasks
                 or "num_distributed_tasks" in ctx.request_params
             ):
-                raise ValueError(
-                    "Distributed execution is only supported in FiftyOne Enterprise"
-                )
+                # Distributed execution is only supported in FiftyOne Enterprise
+                ctx.request_params.pop("num_distributed_tasks", None)
+
             if isinstance(operator, PipelineOperator):
                 raise ValueError(
                     "Pipeline operators require a distributed executor, "
@@ -372,6 +374,7 @@ async def prepare_operator_executor(
     request_params,
     set_progress=None,
     delegated_operation_id=None,
+    pipeline_ctx=None,
 ):
     registry = OperatorRegistry()
     if registry.operator_exists(operator_uri) is False:
@@ -386,6 +389,7 @@ async def prepare_operator_executor(
         delegated_operation_id=delegated_operation_id,
         operator_uri=operator_uri,
         required_secrets=operator._plugin_secrets,
+        pipeline=pipeline_ctx,
     )
 
     await ctx.resolve_secret_values(operator._plugin_secrets)
@@ -521,7 +525,7 @@ class ExecutionContext(object):
     selected samples, as well as to trigger other operators.
 
     Args:
-        request_params (None): a optional dictionary of request parameters
+        request_params (None): an optional dictionary of request parameters
         executor (None): an optional :class:`Executor` instance
         set_progress (None): an optional function to set the progress of the
             current operation
@@ -530,6 +534,9 @@ class ExecutionContext(object):
         operator_uri (None): the unique id of the operator
         required_secrets (None): the list of required secrets from the
             plugin's definition
+        pipeline (None): an optional :class:`PipelineExecutionContext` with
+            information about the current pipeline execution, if this operator
+            is being executed as part of a pipeline
     """
 
     def __init__(
@@ -540,6 +547,7 @@ class ExecutionContext(object):
         delegated_operation_id=None,
         operator_uri=None,
         required_secrets=None,
+        pipeline=None,
     ):
         if request_params is None:
             request_params = {}
@@ -547,6 +555,7 @@ class ExecutionContext(object):
         self.params = self.request_params.get("params", {})
         self.executor = executor
         self.user = None
+        self.pipeline = pipeline
 
         self._dataset = None
         self._view = None
@@ -1097,6 +1106,49 @@ class ExecutionResult(object):
         }
 
 
+@dataclasses.dataclass
+class PipelineExecutionContext(object):
+    """Represents the execution context of a pipeline.
+
+    Operators can use the pipeline execution context to access information
+    about the current pipeline execution, if they are a child operation in a
+    pipeline.
+    """
+
+    active: bool
+    """Whether the pipeline is currently active, i.e., having no failures in
+    prior stages
+    """
+
+    curr_stage_index: int
+    """Index of the pipeline's current execution stage"""
+
+    total_stages: int
+    """The total number of stages in the pipeline"""
+
+    num_distributed_tasks: int = 0
+    """The number of distributed tasks in the current stage"""
+
+    error: Optional[str] = None
+    """Error message, if any. Applicable only if the pipeline is not active"""
+
+    # Overriding default init so we swallow extra kwargs
+    def __init__(
+        self,
+        active,
+        curr_stage_index,
+        total_stages,
+        error=None,
+        num_distributed_tasks=0,
+        **_,
+    ):
+        self.active = active
+        self.curr_stage_index = curr_stage_index
+        self.total_stages = total_stages
+        self.error = error
+        self.num_distributed_tasks = num_distributed_tasks
+
+
 class ExecutionError(Exception):
     """An error that occurs while executing an operator."""
 
