@@ -1,10 +1,11 @@
 """
 FiftyOne Server main
 
-| Copyright 2017-2023, Voxel51, Inc.
+| Copyright 2017-2025, Voxel51, Inc.
 | `voxel51.com <https://voxel51.com/>`_
 |
 """
+
 import argparse
 import os
 
@@ -13,9 +14,6 @@ from hypercorn.asyncio import serve
 from hypercorn.config import Config
 import logging
 
-if os.environ.get("FIFTYONE_ENABLE_DEBUG_LOGGING", False):
-    logging.getLogger("asyncio").setLevel(logging.DEBUG)
-
 if os.environ.get("FIFTYONE_DISABLE_SERVICES", False):
     del os.environ["FIFTYONE_DISABLE_SERVICES"]
 
@@ -27,6 +25,10 @@ import fiftyone.constants as foc
 from fiftyone.server.app import app
 from fiftyone.server.events import set_port
 
+DEBUG_LOGGING = fo.config.logging_level == "DEBUG"
+
+if DEBUG_LOGGING:
+    logging.getLogger("asyncio").setLevel(logging.DEBUG)
 
 if __name__ == "__main__":
     parser = argparse.ArgumentParser()
@@ -34,10 +36,15 @@ if __name__ == "__main__":
     parser.add_argument(
         "--address", type=str, default=fo.config.default_app_address
     )
+    parser.add_argument("--clean_start", action="store_true")
     args = parser.parse_args()
     config = Config()
     config.bind = [f"{args.address}:{args.port}"]
     set_port(args.port)
 
     config.use_reloader = foc.DEV_INSTALL
-    asyncio.run(serve(app, config), debug=foc.DEV_INSTALL)
+
+    if args.clean_start:
+        fo.delete_datasets("*")
+
+    asyncio.run(serve(app, config), debug=DEBUG_LOGGING)
