@@ -1,19 +1,21 @@
 """
 FiftyOne Server /sort route
 
-| Copyright 2017-2023, Voxel51, Inc.
+| Copyright 2017-2025, Voxel51, Inc.
 | `voxel51.com <https://voxel51.com/>`_
 |
 """
-from dataclasses import asdict
 
 from starlette.endpoints import HTTPEndpoint
 from starlette.requests import Request
 
+from fiftyone.core.session.events import StateUpdate
+import fiftyone.core.stages as fos
+
 from fiftyone.server.decorators import route
 import fiftyone.server.events as fose
-from fiftyone.server.query import serialize_dataset
 import fiftyone.server.view as fosv
+from fiftyone.server.filters import GroupElementFilter, SampleFilter
 
 
 class Sort(HTTPEndpoint):
@@ -23,24 +25,32 @@ class Sort(HTTPEndpoint):
         filters = data.get("filters", {})
         stages = data.get("view", None)
         subscription = data.get("subscription", None)
-        fosv.get_view(
+        slice = data.get("slice", None)
+
+        view = await fosv.get_view(
             dataset_name,
             stages=stages,
             filters=filters,
             extended_stages={
                 "fiftyone.core.stages.SortBySimilarity": data["extended"]
             },
+            sample_filter=(
+                SampleFilter(group=GroupElementFilter(slice=slice))
+                if slice is not None
+                else None
+            ),
+            awaitable=True,
         )
+
         state = fose.get_state()
         state.selected = []
         state.selected_labels = []
 
-        await fose.dispatch_event(subscription, fose.StateUpdate(state))
-        return {
-            "dataset": asdict(
-                await serialize_dataset(
-                    dataset_name=dataset_name,
-                    serialized_view=stages,
-                )
-            )
-        }
+        await fose.dispatch_event(subscription, StateUpdate(state))
+
+        # return the new sort by stage
+        for stage in reversed(view._stages):
+            if isinstance(stage, fos.SortBySimilarity):
+                return stage._serialize(include_uuid=False)
+
+        raise ValueError("sorting not found")
