@@ -1,10 +1,11 @@
 """
 COCO-style detection evaluation.
 
-| Copyright 2017-2023, Voxel51, Inc.
+| Copyright 2017-2025, Voxel51, Inc.
 | `voxel51.com <https://voxel51.com/>`_
 |
 """
+
 import logging
 from collections import defaultdict
 
@@ -12,6 +13,7 @@ import numpy as np
 
 import eta.core.utils as etau
 
+import fiftyone.core.labels as fol
 import fiftyone.core.plots as fop
 import fiftyone.utils.iou as foui
 
@@ -30,11 +32,13 @@ class COCOEvaluationConfig(DetectionEvaluationConfig):
 
     Args:
         pred_field: the name of the field containing the predicted
-            :class:`fiftyone.core.labels.Detections` or
-            :class:`fiftyone.core.labels.Polylines`
+            :class:`fiftyone.core.labels.Detections`,
+            :class:`fiftyone.core.labels.Polylines`, or
+            :class:`fiftyone.core.labels.Keypoints`
         gt_field: the name of the field containing the ground truth
-            :class:`fiftyone.core.labels.Detections` or
-            :class:`fiftyone.core.labels.Polylines`
+            :class:`fiftyone.core.labels.Detections`,
+            :class:`fiftyone.core.labels.Polylines`, or
+            :class:`fiftyone.core.labels.Keypoints`
         iou (None): the IoU threshold to use to determine matches
         classwise (None): whether to only match objects with the same class
             label (True) or allow matches between classes (False)
@@ -46,9 +50,10 @@ class COCOEvaluationConfig(DetectionEvaluationConfig):
             of the provided :class:`fiftyone.core.labels.Polyline` instances
             rather than using their actual geometries
         tolerance (None): a tolerance, in pixels, when generating approximate
-            polylines for instance masks. Typical values are 1-3 pixels
+            polylines for instance masks. Typical values are 1-3 pixels. By
+            default, IoUs are computed directly on the dense pixel masks
         compute_mAP (False): whether to perform the necessary computations so
-            that mAP and PR curves can be generated
+            that mAP, mAR, and PR curves can be generated
         iou_threshs (None): a list of IoU thresholds to use when computing mAP
             and PR curves. Only applicable when ``compute_mAP`` is True
         max_preds (None): the maximum number of predicted objects to evaluate
@@ -63,6 +68,8 @@ class COCOEvaluationConfig(DetectionEvaluationConfig):
 
             If ``error_level > 0``, any calculation that raises a geometric
             error will default to an IoU of 0
+        custom_metrics (None): an optional list of custom metrics to compute
+            or dict mapping metric names to kwargs dicts
     """
 
     def __init__(
@@ -79,10 +86,16 @@ class COCOEvaluationConfig(DetectionEvaluationConfig):
         iou_threshs=None,
         max_preds=None,
         error_level=1,
+        custom_metrics=None,
         **kwargs,
     ):
         super().__init__(
-            pred_field, gt_field, iou=iou, classwise=classwise, **kwargs
+            pred_field,
+            gt_field,
+            iou=iou,
+            classwise=classwise,
+            custom_metrics=custom_metrics,
+            **kwargs,
         )
 
         if compute_mAP and iou_threshs is None:
@@ -164,7 +177,13 @@ class COCOEvaluation(DetectionEvaluation):
         return _coco_evaluation_single_iou(gts, preds, eval_key, self.config)
 
     def generate_results(
-        self, samples, matches, eval_key=None, classes=None, missing=None
+        self,
+        samples,
+        matches,
+        eval_key=None,
+        classes=None,
+        missing=None,
+        progress=None,
     ):
         """Generates aggregate evaluation results for the samples.
 
@@ -186,6 +205,9 @@ class COCOEvaluation(DetectionEvaluation):
                 purposes
             missing (None): a missing label string. Any unmatched objects are
                 given this label for results purposes
+            progress (None): whether to render a progress bar (True/False), use
+                the default value ``fiftyone.config.show_progress_bars``
+                (None), or a progress callback function to invoke instead
 
         Returns:
             a :class:`DetectionResults`
@@ -207,7 +229,10 @@ class COCOEvaluation(DetectionEvaluation):
             thresholds,
             iou_threshs,
             classes,
-        ) = _compute_pr_curves(samples, self.config, classes=classes)
+            recall_sweep,
+        ) = _compute_pr_curves(
+            samples, self.config, classes=classes, progress=progress
+        )
 
         return COCODetectionResults(
             samples,
@@ -218,6 +243,7 @@ class COCOEvaluation(DetectionEvaluation):
             recall,
             iou_threshs,
             classes,
+            recall_sweep=recall_sweep,
             thresholds=thresholds,
             missing=missing,
             backend=self,
@@ -240,10 +266,13 @@ class COCODetectionResults(DetectionResults):
         recall: an array of recall values
         iou_threshs: an array of IoU thresholds
         classes: the list of possible classes
+        recall_sweep (None): an array of recall values of shape
+            ``num_iou x num_classes``
         thresholds (None): an optional array of decision thresholds of shape
             ``num_iou_threshs x num_classes x num_recall``
         missing (None): a missing label string. Any unmatched objects are
             given this label for evaluation purposes
+        custom_metrics (None): an optional dict of custom metrics
         backend (None): a :class:`COCOEvaluation` backend
     """
 
@@ -257,8 +286,10 @@ class COCODetectionResults(DetectionResults):
         recall,
         iou_threshs,
         classes,
+        recall_sweep=None,
         thresholds=None,
         missing=None,
+        custom_metrics=None,
         backend=None,
     ):
         super().__init__(
@@ -268,17 +299,22 @@ class COCODetectionResults(DetectionResults):
             matches,
             classes=classes,
             missing=missing,
+            custom_metrics=custom_metrics,
             backend=backend,
         )
 
         self.precision = np.asarray(precision)
         self.recall = np.asarray(recall)
         self.iou_threshs = np.asarray(iou_threshs)
+        self.recall_sweep = recall_sweep
         self.thresholds = (
             np.asarray(thresholds) if thresholds is not None else None
         )
 
         self._classwise_AP = np.mean(precision, axis=(0, 2))
+        self._classwise_AR = (
+            np.mean(recall_sweep, axis=0) if recall_sweep is not None else None
+        )
 
     def plot_pr_curves(
         self, classes=None, iou_thresh=None, backend="plotly", **kwargs
@@ -363,6 +399,36 @@ class COCODetectionResults(DetectionResults):
 
         return np.mean(classwise_AP)
 
+    def mAR(self, classes=None):
+        """Computes COCO-style mean average recall (mAR) for the specified
+        classes.
+
+        See `this page <https://github.com/cocodataset/cocoapi/blob/master/PythonAPI/pycocotools/cocoeval.py>`_
+        for more details about COCO-style mAR.
+
+        Args:
+            classes (None): a list of classes for which to compute mAR
+
+        Returns:
+            the mAR in ``[0, 1]``
+        """
+        if self._classwise_AR is None:
+            raise Exception(
+                "Classwise AR is not available. mAR can't be computed."
+            )
+
+        if classes is not None:
+            class_inds = np.array([self._get_class_index(c) for c in classes])
+            classwise_AR = self._classwise_AR[class_inds]
+        else:
+            classwise_AR = self._classwise_AR
+
+        classwise_AR = classwise_AR[classwise_AR > -1]
+        if classwise_AR.size == 0:
+            return -1
+
+        return np.mean(classwise_AR)
+
     def _get_iou_thresh_inds(self, iou_thresh=None):
         if iou_thresh is None:
             return np.arange(len(self.iou_threshs))
@@ -397,6 +463,7 @@ class COCODetectionResults(DetectionResults):
         precision = d["precision"]
         recall = d["recall"]
         iou_threshs = d["iou_threshs"]
+        recall_sweep = d.get("recall_sweep", None)
         thresholds = d.get("thresholds", None)
         return super()._from_dict(
             d,
@@ -406,6 +473,7 @@ class COCODetectionResults(DetectionResults):
             precision=precision,
             recall=recall,
             iou_threshs=iou_threshs,
+            recall_sweep=recall_sweep,
             thresholds=thresholds,
             **kwargs,
         )
@@ -497,6 +565,11 @@ def _coco_evaluation_setup(
             label = obj.label if classwise else "all"
             cats[label]["preds"].append(obj)
 
+    if isinstance(preds, fol.Keypoints):
+        sort_key = lambda p: np.nanmean(p.confidence) if p.confidence else -1
+    else:
+        sort_key = lambda p: p.confidence or -1
+
     # Compute IoUs within each category
     pred_ious = {}
     for objects in cats.values():
@@ -504,7 +577,7 @@ def _coco_evaluation_setup(
         preds = objects["preds"]
 
         # Highest confidence predictions first
-        preds = sorted(preds, key=lambda p: p.confidence or -1, reverse=True)
+        preds = sorted(preds, key=sort_key, reverse=True)
 
         if max_preds is not None:
             preds = preds[:max_preds]
@@ -514,12 +587,9 @@ def _coco_evaluation_setup(
         # Sort ground truth so crowds are last
         gts = sorted(gts, key=iscrowd)
 
-        # Compute ``num_preds x num_gts`` IoUs
-        ious = foui.compute_ious(preds, gts, **iou_kwargs)
-
-        gt_ids = [g.id for g in gts]
-        for pred, gt_ious in zip(preds, ious):
-            pred_ious[pred.id] = list(zip(gt_ids, gt_ious))
+        # Compute IoUs for overlapping detections
+        ious = foui.compute_ious(preds, gts, sparse=True, **iou_kwargs)
+        pred_ious.update(ious)
 
     return cats, pred_ious, iscrowd
 
@@ -535,6 +605,13 @@ def _compute_matches(
 
         # Match each prediction to the highest available IoU ground truth
         for pred in objects["preds"]:
+            if isinstance(pred, fol.Keypoint):
+                pred_conf = (
+                    np.nanmean(pred.confidence) if pred.confidence else None
+                )
+            else:
+                pred_conf = pred.confidence
+
             if pred.id in pred_ious:
                 best_match = None
                 best_match_iou = iou_thresh
@@ -586,7 +663,7 @@ def _compute_matches(
                             gt.label,
                             pred.label,
                             best_match_iou,
-                            pred.confidence,
+                            pred_conf,
                             gt.id,
                             pred.id,
                             iscrowd(gt),
@@ -599,21 +676,20 @@ def _compute_matches(
                             None,
                             pred.label,
                             None,
-                            pred.confidence,
+                            pred_conf,
                             None,
                             pred.id,
                             None,
                         )
                     )
-
-            elif pred.label == cat:
+            else:
                 pred[eval_key] = "fp"
                 matches.append(
                     (
                         None,
                         pred.label,
                         None,
-                        pred.confidence,
+                        pred_conf,
                         None,
                         pred.id,
                         None,
@@ -631,7 +707,7 @@ def _compute_matches(
     return matches
 
 
-def _compute_pr_curves(samples, config, classes=None):
+def _compute_pr_curves(samples, config, classes=None, progress=None):
     gt_field = config.gt_field
     pred_field = config.pred_field
     iou_threshs = config.iou_threshs
@@ -648,7 +724,7 @@ def _compute_pr_curves(samples, config, classes=None):
         _classes = set()
 
     logger.info("Performing IoU sweep...")
-    for sample in samples.iter_samples(progress=True):
+    for sample in samples.iter_samples(progress=progress):
         if processing_frames:
             images = sample.frames.values()
         else:
@@ -703,6 +779,7 @@ def _compute_pr_curves(samples, config, classes=None):
     precision = -np.ones((num_threshs, num_classes, 101))
     thresholds = -np.ones((num_threshs, num_classes, 101))
     recall = np.linspace(0, 1, 101)
+    recall_sweep = -np.ones((num_threshs, num_classes))
     for idx, _thresh_matches in enumerate(thresh_matches):
         for c, matches in _thresh_matches.items():
             c_idx = class_idx_map.get(c, None)
@@ -750,8 +827,12 @@ def _compute_pr_curves(samples, config, classes=None):
 
             precision[idx][c_idx] = q
             thresholds[idx][c_idx] = t
+            if rec.size != 0:
+                recall_sweep[idx][c_idx] = rec[-1]
+            else:
+                recall_sweep[idx][c_idx] = 0
 
-    return precision, recall, thresholds, iou_threshs, classes
+    return precision, recall, thresholds, iou_threshs, classes, recall_sweep
 
 
 def _copy_labels(labels):
