@@ -1,22 +1,30 @@
 """
 Detection evaluation.
 
-| Copyright 2017-2023, Voxel51, Inc.
+| Copyright 2017-2025, Voxel51, Inc.
 | `voxel51.com <https://voxel51.com/>`_
 |
 """
+from copy import deepcopy
+import inspect
 import itertools
 import logging
 
 import numpy as np
 
-import fiftyone.core.evaluation as foe
+import eta.core.utils as etau
+
+import fiftyone as fo
 import fiftyone.core.fields as fof
 import fiftyone.core.labels as fol
 import fiftyone.core.utils as fou
 import fiftyone.core.validation as fov
 
-from .base import BaseEvaluationResults
+from .base import (
+    BaseEvaluationMethod,
+    BaseEvaluationMethodConfig,
+    BaseClassificationResults,
+)
 
 
 logger = logging.getLogger(__name__)
@@ -35,6 +43,8 @@ def evaluate_detections(
     use_boxes=False,
     classwise=True,
     dynamic=True,
+    custom_metrics=None,
+    progress=None,
     **kwargs,
 ):
     """Evaluates the predicted detections in the given samples with respect to
@@ -46,12 +56,16 @@ def evaluate_detections(
     -   Instance segmentations in :class:`fiftyone.core.labels.Detections`
         format with their ``mask`` attributes populated
     -   Polygons in :class:`fiftyone.core.labels.Polylines` format
+    -   Keypoints in :class:`fiftyone.core.labels.Keypoints` format
     -   Temporal detections in :class:`fiftyone.core.labels.TemporalDetections`
         format
 
     For spatial object detection evaluation, this method uses COCO-style
     evaluation by default.
 
+    When evaluating keypoints, "IoUs" are computed via
+    `object keypoint similarity <https://cocodataset.org/#keypoints-eval>`_.
+
     For temporal segment detection, this method uses ActivityNet-style
     evaluation by default.
 
@@ -59,7 +73,7 @@ def evaluate_detections(
     can optionally customize the method by passing additional parameters for
     the method's config class as ``kwargs``.
 
-    The supported ``method`` values and their associated configs are:
+    The natively provided ``method`` values and their associated configs are:
 
     -   ``"coco"``: :class:`fiftyone.utils.eval.coco.COCOEvaluationConfig`
     -   ``"open-images"``: :class:`fiftyone.utils.eval.openimages.OpenImagesEvaluationConfig`
@@ -95,21 +109,22 @@ def evaluate_detections(
         pred_field: the name of the field containing the predicted
             :class:`fiftyone.core.labels.Detections`,
             :class:`fiftyone.core.labels.Polylines`,
+            :class:`fiftyone.core.labels.Keypoints`,
             or :class:`fiftyone.core.labels.TemporalDetections`
         gt_field ("ground_truth"): the name of the field containing the ground
             truth :class:`fiftyone.core.labels.Detections`,
             :class:`fiftyone.core.labels.Polylines`,
+            :class:`fiftyone.core.labels.Keypoints`,
             or :class:`fiftyone.core.labels.TemporalDetections`
         eval_key (None): an evaluation key to use to refer to this evaluation
         classes (None): the list of possible classes. If not provided, the
             observed ground truth/predicted labels are used
         missing (None): a missing label string. Any unmatched objects are given
             this label for results purposes
-        method (None): a string specifying the evaluation method to use.
-            For spatial object detection, the supported values are
-            ``("coco", "open-images")`` and the default is ``"coco"``. For
-            temporal segment detection, the supported values are
-            ``("activitynet")`` and the default is ``"activitynet"``
+        method (None): a string specifying the evaluation method to use. The
+            supported values are
+            ``fo.evaluation_config.detection_backends.keys()`` and the default
+            is ``fo.evaluation_config.default_detection_backend``
         iou (0.50): the IoU threshold to use to determine matches
         use_masks (False): whether to compute IoUs using the instances masks in
             the ``mask`` attribute of the provided objects, which must be
@@ -121,16 +136,22 @@ def evaluate_detections(
             label (True) or allow matches between classes (False)
         dynamic (True): whether to declare the dynamic object-level attributes
             that are populated on the dataset's schema
+        custom_metrics (None): an optional list of custom metrics to compute
+            or dict mapping metric names to kwargs dicts
+        progress (None): whether to render a progress bar (True/False), use the
+            default value ``fiftyone.config.show_progress_bars`` (None), or a
+            progress callback function to invoke instead
         **kwargs: optional keyword arguments for the constructor of the
             :class:`DetectionEvaluationConfig` being used
 
     Returns:
         a :class:`DetectionResults`
     """
+    fov.validate_non_grouped_collection(samples)
     fov.validate_collection_label_fields(
         samples,
         (pred_field, gt_field),
-        (fol.Detections, fol.Polylines, fol.TemporalDetections),
+        (fol.Detections, fol.Polylines, fol.Keypoints, fol.TemporalDetections),
         same_type=True,
     )
 
@@ -149,6 +170,7 @@ def evaluate_detections(
         is_temporal,
         iou=iou,
         classwise=classwise,
+        custom_metrics=custom_metrics,
         **kwargs,
     )
 
@@ -159,8 +181,9 @@ def evaluate_detections(
     eval_method.register_samples(samples, eval_key, dynamic=dynamic)
 
     processing_frames = samples._is_frame_field(pred_field)
+    save = eval_key is not None
 
-    if eval_key is not None:
+    if save:
         tp_field = "%s_tp" % eval_key
         fp_field = "%s_fp" % eval_key
         fn_field = "%s_fn" % eval_key
@@ -172,7 +195,7 @@ def evaluate_detections(
 
     matches = []
     logger.info("Evaluating detections...")
-    for sample in _samples.iter_samples(progress=True):
+    for sample in _samples.iter_samples(progress=progress, autosave=save):
         if processing_frames:
             docs = sample.frames.values()
         else:
@@ -189,48 +212,73 @@ def evaluate_detections(
             sample_fp += fp
             sample_fn += fn
 
-            if processing_frames and eval_key is not None:
+            if processing_frames and save:
                 doc[tp_field] = tp
                 doc[fp_field] = fp
                 doc[fn_field] = fn
 
-        if eval_key is not None:
+        if save:
             sample[tp_field] = sample_tp
             sample[fp_field] = sample_fp
             sample[fn_field] = sample_fn
-            sample.save()
 
     results = eval_method.generate_results(
-        samples, matches, eval_key=eval_key, classes=classes, missing=missing
+        samples,
+        matches,
+        eval_key=eval_key,
+        classes=classes,
+        missing=missing,
+        progress=progress,
     )
-    eval_method.save_run_results(samples, eval_key, results)
+    eval_method.compute_custom_metrics(samples, eval_key, results)
+
+    if eval_key is not None:
+        eval_method.save_run_results(samples, eval_key, results)
+        eval_method.add_fields_to_sidebar_group(
+            samples, eval_key, omit_fields=(pred_field, gt_field)
+        )
 
     return results
 
 
-class DetectionEvaluationConfig(foe.EvaluationMethodConfig):
+class DetectionEvaluationConfig(BaseEvaluationMethodConfig):
     """Base class for configuring :class:`DetectionEvaluation` instances.
 
     Args:
         pred_field: the name of the field containing the predicted
-            :class:`fiftyone.core.labels.Detections` or
-            :class:`fiftyone.core.labels.Polylines`
+            :class:`fiftyone.core.labels.Detections`,
+            :class:`fiftyone.core.labels.Polylines`, or
+            :class:`fiftyone.core.labels.Keypoints`
         gt_field: the name of the field containing the ground truth
-            :class:`fiftyone.core.labels.Detections` or
-            :class:`fiftyone.core.labels.Polylines`
+            :class:`fiftyone.core.labels.Detections`,
+            :class:`fiftyone.core.labels.Polylines`, or
+            :class:`fiftyone.core.labels.Keypoints`
         iou (None): the IoU threshold to use to determine matches
         classwise (None): whether to only match objects with the same class
             label (True) or allow matches between classes (False)
+        custom_metrics (None): an optional list of custom metrics to compute
+            or dict mapping metric names to kwargs dicts
     """
 
     def __init__(
-        self, pred_field, gt_field, iou=None, classwise=None, **kwargs
+        self,
+        pred_field,
+        gt_field,
+        iou=None,
+        classwise=None,
+        custom_metrics=None,
+        **kwargs,
     ):
         super().__init__(**kwargs)
         self.pred_field = pred_field
         self.gt_field = gt_field
         self.iou = iou
         self.classwise = classwise
+        self.custom_metrics = custom_metrics
+
+    @property
+    def type(self):
+        return "detection"
 
     @property
     def requires_additional_fields(self):
@@ -244,7 +292,7 @@ class DetectionEvaluationConfig(foe.EvaluationMethodConfig):
         return False
 
 
-class DetectionEvaluation(foe.EvaluationMethod):
+class DetectionEvaluation(BaseEvaluationMethod):
     """Base class for detection evaluation methods.
 
     Args:
@@ -343,7 +391,13 @@ class DetectionEvaluation(foe.EvaluationMethod):
         raise NotImplementedError("subclass must implement evaluate()")
 
     def generate_results(
-        self, samples, matches, eval_key=None, classes=None, missing=None
+        self,
+        samples,
+        matches,
+        eval_key=None,
+        classes=None,
+        missing=None,
+        progress=None,
     ):
         """Generates aggregate evaluation results for the samples.
 
@@ -362,6 +416,9 @@ class DetectionEvaluation(foe.EvaluationMethod):
                 purposes
             missing (None): a missing label string. Any unmatched objects are
                 given this label for results purposes
+            progress (None): whether to render a progress bar (True/False), use
+                the default value ``fiftyone.config.show_progress_bars``
+                (None), or a progress callback function to invoke instead
 
         Returns:
             a :class:`DetectionResults`
@@ -376,7 +433,7 @@ class DetectionEvaluation(foe.EvaluationMethod):
             backend=self,
         )
 
-    def get_fields(self, samples, eval_key):
+    def get_fields(self, samples, eval_key, include_custom_metrics=True):
         pred_field = self.config.pred_field
         pred_type = samples._get_label_field_type(pred_field)
         pred_key = "%s.%s.%s" % (
@@ -407,13 +464,20 @@ class DetectionEvaluation(foe.EvaluationMethod):
                 ["%s_tp" % prefix, "%s_fp" % prefix, "%s_fn" % prefix]
             )
 
+        if include_custom_metrics:
+            fields.extend(self.get_custom_metric_fields(samples, eval_key))
+
         return fields
 
     def rename(self, samples, eval_key, new_eval_key):
         dataset = samples._dataset
 
-        in_fields = self.get_fields(dataset, eval_key)
-        out_fields = self.get_fields(dataset, new_eval_key)
+        in_fields = self.get_fields(
+            dataset, eval_key, include_custom_metrics=False
+        )
+        out_fields = self.get_fields(
+            dataset, new_eval_key, include_custom_metrics=False
+        )
 
         in_sample_fields, in_frame_fields = fou.split_frame_fields(in_fields)
         out_sample_fields, out_frame_fields = fou.split_frame_fields(
@@ -428,6 +492,10 @@ class DetectionEvaluation(foe.EvaluationMethod):
             fields = dict(zip(in_frame_fields, out_frame_fields))
             dataset.rename_frame_fields(fields)
 
+        self.rename_custom_metrics(samples, eval_key, new_eval_key)
+
+        samples._rename_sidebar_group(eval_key, new_eval_key)
+
     def cleanup(self, samples, eval_key):
         dataset = samples._dataset
 
@@ -472,12 +540,16 @@ class DetectionEvaluation(foe.EvaluationMethod):
         else:
             dataset.delete_sample_fields(fields, error_level=1)
 
+        self.cleanup_custom_metrics(samples, eval_key)
+
+        samples._delete_empty_sidebar_group(eval_key)
+
     def _validate_run(self, samples, eval_key, existing_info):
         self._validate_fields_match(eval_key, "pred_field", existing_info)
         self._validate_fields_match(eval_key, "gt_field", existing_info)
 
 
-class DetectionResults(BaseEvaluationResults):
+class DetectionResults(BaseClassificationResults):
     """Class that stores the results of a detection evaluation.
 
     Args:
@@ -492,6 +564,7 @@ class DetectionResults(BaseEvaluationResults):
             observed ground truth/predicted labels are used
         missing (None): a missing label string. Any unmatched objects are given
             this label for evaluation purposes
+        custom_metrics (None): an optional dict of custom metrics
         backend (None): a :class:`DetectionEvaluation` backend
     """
 
@@ -503,6 +576,7 @@ class DetectionResults(BaseEvaluationResults):
         matches,
         classes=None,
         missing=None,
+        custom_metrics=None,
         backend=None,
     ):
         if matches:
@@ -528,6 +602,7 @@ class DetectionResults(BaseEvaluationResults):
             ypred_ids=ypred_ids,
             classes=classes,
             missing=missing,
+            custom_metrics=custom_metrics,
             backend=backend,
         )
 
@@ -553,6 +628,7 @@ class DetectionResults(BaseEvaluationResults):
 
         classes = d.get("classes", None)
         missing = d.get("missing", None)
+        custom_metrics = d.get("custom_metrics", None)
 
         matches = list(zip(ytrue, ypred, ious, confs, ytrue_ids, ypred_ids))
 
@@ -563,6 +639,7 @@ class DetectionResults(BaseEvaluationResults):
             matches,
             classes=classes,
             missing=missing,
+            custom_metrics=custom_metrics,
             **kwargs,
         )
 
@@ -570,26 +647,43 @@ class DetectionResults(BaseEvaluationResults):
 def _parse_config(pred_field, gt_field, method, is_temporal, **kwargs):
     if method is None:
         if is_temporal:
+            # @todo expose this in `fo.evaluation_config`?
             method = "activitynet"
         else:
-            method = "coco"
+            method = fo.evaluation_config.default_detection_backend
 
-    if method == "activitynet":
-        from .activitynet import ActivityNetEvaluationConfig
+    custom_metrics = kwargs.get("custom_metrics", None)
+    if etau.is_str(custom_metrics):
+        kwargs["custom_metrics"] = [custom_metrics]
 
-        return ActivityNetEvaluationConfig(pred_field, gt_field, **kwargs)
+    if inspect.isclass(method):
+        return method(pred_field, gt_field, **kwargs)
 
-    if method == "coco":
-        from .coco import COCOEvaluationConfig
+    backends = fo.evaluation_config.detection_backends
 
-        return COCOEvaluationConfig(pred_field, gt_field, **kwargs)
+    if method not in backends:
+        raise ValueError(
+            "Unsupported detection evaluation method '%s'. The available "
+            "methods are %s" % (method, sorted(backends.keys()))
+        )
 
-    if method == "open-images":
-        from .openimages import OpenImagesEvaluationConfig
+    params = deepcopy(backends[method])
 
-        return OpenImagesEvaluationConfig(pred_field, gt_field, **kwargs)
+    config_cls = kwargs.pop("config_cls", None)
 
-    raise ValueError("Unsupported evaluation method '%s'" % method)
+    if config_cls is None:
+        config_cls = params.pop("config_cls", None)
+
+    if config_cls is None:
+        raise ValueError(
+            "Detection evaluation method '%s' has no `config_cls`" % method
+        )
+
+    if etau.is_str(config_cls):
+        config_cls = etau.get_class(config_cls)
+
+    params.update(**kwargs)
+    return config_cls(pred_field, gt_field, **params)
 
 
 def _tally_matches(matches):
