@@ -1,7 +1,7 @@
 """
 GeoJSON utilities.
 
-| Copyright 2017-2023, Voxel51, Inc.
+| Copyright 2017-2025, Voxel51, Inc.
 | `voxel51.com <https://voxel51.com/>`_
 |
 """
@@ -14,6 +14,7 @@ import eta.core.utils as etau
 import fiftyone.core.fields as fof
 import fiftyone.core.labels as fol
 from fiftyone.core.sample import Sample
+import fiftyone.core.storage as fos
 import fiftyone.core.utils as fou
 import fiftyone.core.validation as fov
 import fiftyone.utils.data as foud
@@ -23,7 +24,11 @@ logger = logging.getLogger(__name__)
 
 
 def load_location_data(
-    samples, geojson_or_path, location_field=None, skip_missing=True
+    samples,
+    geojson_or_path,
+    location_field=None,
+    skip_missing=True,
+    progress=None,
 ):
     """Loads geolocation data for the given samples from the given GeoJSON
     data.
@@ -85,6 +90,9 @@ def load_location_data(
             used, else a new "location" field is created
         skip_missing (True): whether to skip GeoJSON features with no
             ``filename`` properties (True) or raise an error (False)
+        progress (None): whether to render a progress bar (True/False), use the
+            default value ``fiftyone.config.show_progress_bars`` (None), or a
+            progress callback function to invoke instead
     """
     if location_field is None:
         try:
@@ -134,15 +142,20 @@ def load_location_data(
         logger.info("No matching location data found")
         return
 
-    logger.info("Loading location data for %d samples...", len(found_keys))
-    _samples = samples.select_fields(location_field)
-    with fou.ProgressBar() as pb:
-        for key in pb(found_keys):
-            sample_id = lookup[key]
-            geometry = geometries[key]
-            sample = _samples[sample_id]
-            sample[location_field] = location_cls.from_geo_json(geometry)
-            sample.save()
+    locations = {}
+
+    try:
+        logger.info("Loading location data...")
+        with fou.ProgressBar(progress=progress) as pb:
+            for key in pb(found_keys):
+                sample_id = lookup[key]
+                geometry = geometries[key]
+                locations[sample_id] = location_cls.from_geo_json(geometry)
+    finally:
+        logger.info("Saving location data...")
+        samples.set_values(
+            location_field, locations, key_field="id", progress=progress
+        )
 
 
 def to_geo_json_geometry(label):
@@ -455,7 +468,7 @@ class GeoJSONDatasetImporter(
             for feature in geojson.get("features", []):
                 properties = feature["properties"]
                 if "filename" in properties:
-                    filename = fou.normpath(properties.pop("filename"))
+                    filename = fos.normpath(properties.pop("filename"))
                     if os.path.isabs(filename):
                         filepath = filename
                     else:
@@ -553,7 +566,7 @@ class GeoJSONDatasetExporter(
             generate an output path for each exported media. This argument
             allows for populating nested subdirectories that match the shape of
             the input paths. The path is converted to an absolute path (if
-            necessary) via :func:`fiftyone.core.utils.normalize_path`
+            necessary) via :func:`fiftyone.core.storage.normalize_path`
         abs_paths (False): whether to store absolute paths to the images in the
             exported labels
         image_format (None): the image format to use when writing in-memory
