@@ -2,7 +2,7 @@
 Utilities for working with the
 `Kinetics dataset <https://deepmind.com/research/open-source/kinetics>`.
 
-| Copyright 2017-2023, Voxel51, Inc.
+| Copyright 2017-2025, Voxel51, Inc.
 | `voxel51.com <https://voxel51.com/>`_
 |
 """
@@ -48,9 +48,8 @@ def download_kinetics_split(
         classes (None): a string or list of strings specifying required classes
             to load. If provided, only samples containing at least one instance
             of a specified class will be loaded
-        num_workers (None): the number of processes to use when downloading
-            individual video. By default, ``multiprocessing.cpu_count()`` is
-            used
+        num_workers (None): a suggested number of threads to use when
+            downloading individual videos
         shuffle (False): whether to randomly shuffle the order in which samples
             are chosen for partial downloads
         seed (None): a random seed to use when shuffling
@@ -248,6 +247,11 @@ class KineticsDatasetManager(object):
 
         split = self.info.split
         for e, videos in download_errors.items():
+            if not isinstance(e, (str, int, float, bool)) and e != None:
+                try:
+                    e = str(e)
+                except:
+                    e = "Cannot parse error message"
             if e in prev_errors[split]:
                 prev_errors[split][e].extend(videos)
                 prev_errors[split][e] = sorted(set(prev_errors[split][e]))
