@@ -1,8 +1,8 @@
 """
 Utilities for working with annotations in
-`Scale AI format <https://docs.scale.com/reference#annotation>`_.
+`Scale AI format <https://docs.scale.com/reference/introduction>`_.
 
-| Copyright 2017-2023, Voxel51, Inc.
+| Copyright 2017-2025, Voxel51, Inc.
 | `voxel51.com <https://voxel51.com/>`_
 |
 """
@@ -20,11 +20,13 @@ import eta.core.serial as etas
 import eta.core.utils as etau
 import eta.core.web as etaw
 
+import fiftyone.core.collections as foc
 import fiftyone.core.labels as fol
 import fiftyone.core.media as fomm
 import fiftyone.core.metadata as fom
 import fiftyone.core.utils as fou
 import fiftyone.core.validation as fov
+import fiftyone.utils.image as foui
 
 
 logger = logging.getLogger(__name__)
@@ -35,16 +37,17 @@ def import_from_scale(
     labels_dir_or_json,
     label_prefix=None,
     scale_id_field="scale_id",
+    progress=None,
 ):
     """Imports the Scale AI labels into the FiftyOne dataset.
 
     This method supports importing annotations from the following Scale API
     endpoints:
 
-    -   `General Image Annotation <https://docs.scale.com/reference#general-image-annotation>`_
-    -   `Semantic Segmentation Annotation <https://docs.scale.com/reference#semantic-segmentation-annotation>`_
-    -   `General Video Annotation <https://docs.scale.com/reference#general-video-annotation>`_
-    -   `Video Playback <https://docs.scale.com/reference#video-playback>`_
+    -   `General Image Annotation <https://docs.scale.com/reference/general-image-annotation>`_
+    -   `Semantic Segmentation Annotation <https://docs.scale.com/reference/semantic-segmentation-annotation>`_
+    -   `General Video Annotation <https://docs.scale.com/reference/general-video-annotation>`_
+    -   `Video Playback <https://docs.scale.com/reference/video-playback>`_
 
     The ``scale_id_field`` of the FiftyOne samples are used to associate
     samples with their corresponding Scale task IDs.
@@ -80,14 +83,14 @@ def import_from_scale(
 
     The contents of the ``response`` field should be as follows:
 
-    -   `General Image Annotation <https://docs.scale.com/reference#general-image-annotation>`_::
+    -   `General Image Annotation <https://docs.scale.com/reference/general-image-annotation>`_::
 
             {
                 "annotations": [...]
                 "global_attributes": {...}
             }
 
-    -   `Semantic Segmentation Annotation <https://docs.scale.com/reference#semantic-segmentation-annotation>`_::
+    -   `Semantic Segmentation Annotation <https://docs.scale.com/reference/semantic-segmentation-annotation>`_::
 
             {
                 "annotations": {
@@ -105,7 +108,7 @@ def import_from_scale(
         case the mask is downloaded from the web, or the path to the mask on
         disk.
 
-    -   `General Video Annotation <https://docs.scale.com/reference#general-video-annotation>`_::
+    -   `General Video Annotation <https://docs.scale.com/reference/general-video-annotation>`_::
 
             {
                 "annotations": {
@@ -150,7 +153,7 @@ def import_from_scale(
                 "events": [...]
             }
 
-    -   `Video Playback <https://docs.scale.com/reference#video-playback>`_::
+    -   `Video Playback <https://docs.scale.com/reference/video-playback>`_::
 
             {
                 "annotations": {
@@ -185,6 +188,9 @@ def import_from_scale(
             that are created, separated by an underscore
         scale_id_field ("scale_id"): the sample field to use to associate Scale
             task IDs with FiftyOne samples
+        progress (None): whether to render a progress bar (True/False), use the
+            default value ``fiftyone.config.show_progress_bars`` (None), or a
+            progress callback function to invoke instead
     """
     fov.validate_collection(dataset, media_type=(fomm.IMAGE, fomm.VIDEO))
     is_video = dataset.media_type == fomm.VIDEO
@@ -202,7 +208,9 @@ def import_from_scale(
     else:
         label_key = lambda k: k
 
-    with fou.ProgressBar(total=len(labels)) as pb:
+    pb = fou.ProgressBar(total=len(labels), progress=progress)
+    ctx = foc.SaveContext(dataset)
+    with pb, ctx:
         for task_id, task_labels in pb(labels.items()):
             if task_id not in id_map:
                 logger.info(
@@ -241,7 +249,7 @@ def import_from_scale(
                     {label_key(k): v for k, v in labels_dict.items()}
                 )
 
-            sample.save()
+            ctx.save(sample)
 
 
 # @todo add support for `duration_time`, `frame_rate`, and `start_time`
@@ -254,15 +262,16 @@ def export_to_scale(
     video_playback=False,
     label_field=None,
     frame_labels_field=None,
+    progress=None,
 ):
     """Exports labels from the FiftyOne samples to Scale AI format.
 
     This function is useful for generating pre-annotations that can be provided
     to Scale AI via the ``hypothesis`` parameter of the following endpoints:
 
-    -   `General Image Annotation <https://docs.scale.com/reference#general-image-annotation>`_
-    -   `General Video Annotation <https://docs.scale.com/reference#general-video-annotation>`_
-    -   `Video Playback <https://docs.scale.com/reference#video-playback>`_
+    -   `General Image Annotation <https://docs.scale.com/reference/general-image-annotation>`_
+    -   `General Video Annotation <https://docs.scale.com/reference/general-video-annotation>`_
+    -   `Video Playback <https://docs.scale.com/reference/video-playback>`_
 
     The output ``json_path`` will be a JSON file in the following format::
 
@@ -306,7 +315,7 @@ def export_to_scale(
             ...
 
     When ``video_playback == False``, the per-sample JSON files are written in
-    `General Video Annotation format <https://docs.scale.com/reference#general-video-annotation>`_::
+    `General Video Annotation format <https://docs.scale.com/reference/general-video-annotation>`_::
 
         [
             {
@@ -324,7 +333,7 @@ def export_to_scale(
     of the video.
 
     When ``video_playback == True``, the per-sample JSON files are written in
-    `Video Playback format <https://docs.scale.com/reference#video-playback>`_::
+    `Video Playback format <https://docs.scale.com/reference/video-playback>`_::
 
         {
             "annotations": {...}
@@ -352,7 +361,7 @@ def export_to_scale(
             ...
 
     where each per-sample JSON file contains the
-    `events in the video <https://docs.scale.com/reference#events>`_::
+    `events in the video <https://docs.scale.com/reference/events>`_::
 
         {
             "events": [...]
@@ -368,9 +377,9 @@ def export_to_scale(
             events. Only applicable for video datasets
         video_playback (False): whether to export video labels in a suitable
             format for use with the
-            `Video Playback <https://docs.scale.com/reference#video-playback>`_ task.
+            `Video Playback <https://docs.scale.com/reference/video-playback>`_ task.
             By default, video labels are exported for in a suitable format for
-            the `General Video Annotation <https://docs.scale.com/reference#general-video-annotation>`_
+            the `General Video Annotation <https://docs.scale.com/reference/general-video-annotation>`_
             task. Only applicable for video datasets
         label_field (None): optional label field(s) to export. Can be any of
             the following:
@@ -392,6 +401,9 @@ def export_to_scale(
                 when constructing the exported frame labels
 
             By default, no frame labels are exported
+        progress (None): whether to render a progress bar (True/False), use the
+            default value ``fiftyone.config.show_progress_bars`` (None), or a
+            progress callback function to invoke instead
     """
     fov.validate_collection(
         sample_collection, media_type=(fomm.IMAGE, fomm.VIDEO)
@@ -423,12 +435,12 @@ def export_to_scale(
                 "when exporting labels for video datasets"
             )
 
-    # Compute metadata if necessary
     sample_collection.compute_metadata()
 
     # Export the labels
     labels = {}
-    for sample in sample_collection.iter_samples(progress=True):
+    anno_dict = {}
+    for sample in sample_collection.iter_samples(progress=progress):
         metadata = sample.metadata
 
         # Get frame size
@@ -455,8 +467,6 @@ def export_to_scale(
                     frames, frame_size, make_events=make_events
                 )
 
-            anno_dict = {}
-
             # Write annotations
             if video_labels_dir:
                 anno_path = os.path.join(video_labels_dir, sample.id + ".json")
@@ -559,7 +569,7 @@ def _get_frame_labels(sample, frame_label_fields):
     return frames
 
 
-# https://docs.scale.com/reference#general-image-annotation
+# https://docs.scale.com/reference/general-image-annotation
 def _to_scale_image_labels(labels_dict, frame_size):
     annotations = []
     global_attributes = {}
@@ -588,7 +598,7 @@ def _to_scale_image_labels(labels_dict, frame_size):
     return anno_dict
 
 
-# https://docs.scale.com/reference#general-video-annotation
+# https://docs.scale.com/reference/general-video-annotation
 def _to_scale_video_annotation_labels(frames, frame_size, make_events=False):
     in_progress_events = {}
     events = []
@@ -611,7 +621,7 @@ def _to_scale_video_annotation_labels(frames, frame_size, make_events=False):
     return annotations, {"events": events}
 
 
-# https://docs.scale.com/reference#video-playback
+# https://docs.scale.com/reference/video-playback
 def _to_scale_video_playback_labels(frames, frame_size, make_events=False):
     in_progress_events = {}
     events = []
@@ -698,7 +708,7 @@ def _finalize_events(events, frame_number, in_progress_events):
             del in_progress_events[event_label]
 
 
-# https://docs.scale.com/reference#events
+# https://docs.scale.com/reference/events
 def _make_event(label, start, end):
     if end == start:
         return {
@@ -715,7 +725,7 @@ def _make_event(label, start, end):
     }
 
 
-# https://docs.scale.com/reference#video-playback
+# https://docs.scale.com/reference/video-playback
 def _init_video_box(label):
     return {
         "label": label,
@@ -724,7 +734,7 @@ def _init_video_box(label):
     }
 
 
-# https://docs.scale.com/reference#video-playback
+# https://docs.scale.com/reference/video-playback
 def _make_video_box_frame(detection, frame_number, frame_size):
     x, y, w, h = detection.bounding_box
     width, height = frame_size
@@ -743,12 +753,12 @@ def _make_video_box_frame(detection, frame_number, frame_size):
     return frame_dict
 
 
-# https://docs.scale.com/reference#global-attributes
+# https://docs.scale.com/reference/global-attributes
 def _to_classification(name, label):
     return {name: label.label}
 
 
-# https://docs.scale.com/reference#attributes-overview
+# https://docs.scale.com/reference/attributes-overview
 def _get_attributes(label):
     attrs = {}
     for name, value in label.iter_attributes():
@@ -761,7 +771,7 @@ def _get_attributes(label):
     return attrs
 
 
-# https://docs.scale.com/reference#boxes
+# https://docs.scale.com/reference/boxes
 def _to_detections(label, frame_size):
     if isinstance(label, fol.Detections):
         detections = label.detections
@@ -782,8 +792,8 @@ def _to_detections(label, frame_size):
     return annos
 
 
-# https://docs.scale.com/reference#polygons
-# https://docs.scale.com/reference#line
+# https://docs.scale.com/reference/polygons
+# https://docs.scale.com/reference/line
 def _to_polylines(label, frame_size):
     if isinstance(label, fol.Polylines):
         polylines = label.polylines
@@ -805,7 +815,7 @@ def _to_polylines(label, frame_size):
     return annos
 
 
-# https://docs.scale.com/reference#points
+# https://docs.scale.com/reference/points
 def _to_points(label, frame_size):
     if isinstance(label, fol.Keypoints):
         keypoints = label.keypoints
@@ -871,7 +881,7 @@ def _parse_video_labels(task_labels, metadata):
     return _parse_video_annotation_labels(annos, events, metadata, task=task)
 
 
-# https://docs.scale.com/reference#general-video-annotation
+# https://docs.scale.com/reference/general-video-annotation
 def _parse_video_annotation_labels(annos, events, metadata, task=None):
     if annos is not None:
         annos = annos.get("annotations", [])
@@ -938,7 +948,7 @@ def _get_frame_numbers(task, metadata):
     )
 
 
-# https://docs.scale.com/reference#video-playback
+# https://docs.scale.com/reference/video-playback
 def _parse_video_playback_labels(annos, events, metadata):
     frames = defaultdict(dict)
 
@@ -1015,7 +1025,7 @@ def _parse_video_playback_objects(frames, annotations, metadata):
         )
 
 
-# https://docs.scale.com/reference#general-image-annotation
+# https://docs.scale.com/reference/general-image-annotation
 def _parse_image_labels(anno_dict, frame_size):
     labels = {}
 
@@ -1032,7 +1042,14 @@ def _parse_image_labels(anno_dict, frame_size):
 
 
 def _parse_classifications(attrs):
-    return {k: fol.Classification(label=v) for k, v in attrs.items()}
+    classifications = {}
+    for k, v in attrs.items():
+        if isinstance(v, list):
+            classes = [fol.Classification(label=label) for label in v]
+            classifications[k] = fol.Classifications(classifications=classes)
+        else:
+            classifications[k] = fol.Classification(label=v)
+    return classifications
 
 
 def _parse_objects(anno_dict, frame_size):
@@ -1051,7 +1068,6 @@ def _parse_objects(anno_dict, frame_size):
         attributes = anno.get("attributes", {})
 
         if type_ == "box":
-            # Detection
             bounding_box = _parse_bbox(anno, frame_size)
             detections.append(
                 fol.Detection(
@@ -1059,7 +1075,6 @@ def _parse_objects(anno_dict, frame_size):
                 )
             )
         elif type_ == "polygon":
-            # Polyline
             points = _parse_points(anno["vertices"], frame_size)
             polylines.append(
                 fol.Polyline(
@@ -1071,19 +1086,17 @@ def _parse_objects(anno_dict, frame_size):
                 )
             )
         elif type_ == "line":
-            # Polyline
             points = _parse_points(anno["vertices"], frame_size)
             polylines.append(
                 fol.Polyline(
                     label=label,
                     points=[points],
-                    closed=True,
+                    closed=False,
                     filled=False,
                     **attributes,
                 )
             )
         elif type_ == "point":
-            # Polyline
             point = _parse_point(anno, frame_size)
             keypoints.append(
                 fol.Keypoint(label=label, points=[point], **attributes)
@@ -1133,7 +1146,7 @@ def _parse_point(vertex, frame_size):
     return (vertex["x"] / width, vertex["y"] / height)
 
 
-# https://docs.scale.com/reference#segmentannotation-callback-format
+# https://docs.scale.com/reference/segmentannotation-callback-format
 def _parse_mask(anno_dict):
     indexed_image_uri = anno_dict["annotations"]["combined"]["indexedImage"]
     mask = _download_or_load_image(indexed_image_uri)
@@ -1168,4 +1181,4 @@ def _download_or_load_image(url_or_filepath):
         img_bytes = etaw.download_file(url_or_filepath, quiet=True)
         return etai.decode(img_bytes)
 
-    return etai.read(url_or_filepath)
+    return foui.read(url_or_filepath)
